function! text#open_bibtex_key() abort
  let s:uri = split(split(expand('<cWORD>'), '{')[1], ',')[0]
  if s:uri !=# ''
    silent exec "!xdg-open ~/Documents/papers/'".s:uri."'.pdf"
  else
    echo 'No match found.'
  endif
endfunction

"" Update Emacs timestamp
function! text#last_modified() abort
  if &modified
    let save_cursor = getpos('.')
    let n = min([20, line('$')])
    keepjumps exe '1,' . n . 's#^\(.\{,10}Time-stamp: \).*#\1' .
          \ strftime('<%Y-%m-%d %H:%M:%S chl>') . '#e'
    call histdel('search', -1)
    call setpos('.', save_cursor)
  endif
endfun

"" https://vim.fandom.com/wiki/Surround_selection_with_text
function! text#surround(s1, s2) range
  exe "normal vgvmboma\<Esc>"
  normal! `a
  let lineA = line('.')
  let columnA = col('.')
  normal! `b
  let lineB = line('.')
  let columnB = col('.')
  if lineA > lineB || lineA <= lineB && columnA > columnB
    normal! mc
    normal! `amb
    normal! `cma
  endif
  exe "normal `ba" . a:s2 . "\<Esc>`ai" . a:s1 . "\<Esc>"
endfunction

"" Text alignment
"" https://github.com/strager/dotfiles
"" http://vim.wikia.com/wiki/Regex-based_text_alignment
function! text#align_section(regex) range abort
  let extra = 1
  let sep = empty(a:regex) ? '=' : a:regex
  let maxpos = 0
  let section = getline(a:firstline, a:lastline)
  for line in section
    let pos = match(line, ' *'.sep)
    if maxpos < pos
      let maxpos = pos
    endif
  endfor
  call map(section, 'text#align_line(v:val, sep, maxpos, extra)')
  call setline(a:firstline, section)
endfunction

function! text#align_line(line, sep, maxpos, extra) abort
  let m = matchlist(a:line, '\(.\{-}\) \{-}\('.a:sep.'.*\)')
  if empty(m)
    return a:line
  endif
  let spaces = repeat(' ', a:maxpos - strlen(m[1]) + a:extra)
  return m[1] . spaces . m[2]
endfunction

