local utils = require 'utils'

vim.g.mapleader = " "
vim.g.maplocalleader = ","

-- navigation
utils.map('c', '<C-a>', '<Home>')                                        -- go to beginning of line (cmd)
utils.map('c', '<C-e>', '<End>')                                         -- go to end of line (cmd)
utils.map('i', '<C-a>', '<C-o>0')                                        -- go to beginning of line (insert)
utils.map('i', '<C-e>', '<C-o>$')                                        -- go to end of line (insert)
utils.map('n', '<C-down>', '}')                                          -- move to next blank line
utils.map('n', '<C-up>', '{')                                            -- move to previous blank line

-- buffers
utils.map('n', '<Backspace>', '<C-^>')                                   -- go back to last edited file
utils.map('n', '<localleader>x', '<cmd>bp <bar> bd! #<CR>')              -- close current buffer
utils.map('n', '-', ':Ex<CR>')                                           -- open Netrw
utils.map('n', '<leader>$', '<cmd>tabe +te<CR>')                         -- open terminal in new tab
vim.cmd[[nnoremap <leader>E :tabe <C-R>=expand("%:p:h") . "/" <CR>]]     -- open in new tab in current working directory
vim.cmd[[nnoremap <leader>e :e <C-R>=expand("%:p:h") . "/" <CR>]]        -- open in new window in current working directory
vim.cmd[[nnoremap <silent> <leader>. :lcd %:p:h<CR>]]                    -- set current working directory

-- tabs
utils.map('n', '<leader><tab>', '<cmd>$tabnew<CR>')                      -- open new tab
utils.map('n', '<localleader><tab>', '<cmd>tabnext<CR>')                              -- cycle through tabs

-- block motion
utils.map('n', 'J', 'mzJ`z')                                             -- join lines, but keep cursor stationary
utils.map('n', 'S', 'i<cr><esc>^mwgk:silent! s/\v +$//<cr>:noh<cr>`w')   -- split lines (reciprocal of join, J)
utils.map('n', 'U', '<C-r>')                                             -- redo
utils.map('n', 'gV', '`[v`]')                                            -- reselect the last visual block
utils.map('n', 'zs', '1z=')                                              -- spell fix using first entry
utils.map('v', 'J', ":m '>+1<CR>gv=gv")                                  -- visual block movement (up)
utils.map('v', 'K', ":m '<-2<CR>gv=gv")                                  -- visual block movement (down)
utils.map('i', '<S-Tab>', '<Esc><<i')                                    -- fix line indentation (insert)
utils.map('v', '<', '<gv')                                               -- fix block indentation backward (visual)
utils.map('v', '>', '>gv')                                               -- fix block indentation forward (visual)

-- quickfix
utils.map('n', '<leader>x', '<cmd>cclose <bar> lclose<CR>')              -- close popup windows
utils.map('n', '<leader>X', '<cmd>copen<CR>')                            -- open quickfix window
utils.map('', '<C-j>', '<cmd>:cnext<CR>')                                -- go to next quickfix item
utils.map('', '<C-k>', '<cmd>:cprev<CR>')                                -- got to previous quickfix item
utils.map('n', ']q', '<cmd>cnext<CR>')                                   -- quickfix and loclist navigation
utils.map('n', '[q', '<cmd>cprev<CR>')
utils.map('n', ']Q', '<cmd>clast<CR>')
utils.map('n', '[Q', '<cmd>cfirst<CR>')
utils.map('n', ']l', '<cmd>lnext<CR>')
utils.map('n', '[l', '<cmd>lprev<CR>')
utils.map('n', ']L', '<cmd>llast<CR>')
utils.map('n', '[L', '<cmd>lfirst<CR>')

-- misc
utils.map('n', '<C-l>', '<cmd>nohlsearch<CR>')                                                    -- clear search highlights
utils.map('n', 'n', 'nzz')                                                                        -- keep cursor centered on screen when looking ahead
utils.map('n', 'N', 'Nzz')                                                                        -- keep cursor centered on screen when looking behind
utils.map('n', '<leader>d', '<cmd>call spelling#switch_dict()<CR>')                               -- switch dictionnaries (EN, FR)
utils.map('n', '<localleader>s', 'vip:sort u<CR>')                                                -- sort in lexicographic order (Credit: strager)
utils.map('n', '<localleader>S', 'vip:sort iu<CR>')                                               -- sort in reverse lexicographic order
utils.map('c', 'W!!', 'w !sudo tee % >/dev/null<CR>:e!<CR><CR>')                                  -- write file as root
vim.cmd[[nnoremap <silent> <leader>s :let @s='\<'.expand('<cword>').'\>'<CR>:%s/<C-r>s//<Left>]]  -- substitute current word
vim.cmd[[xnoremap <silent> <leader>s "sy:%s/<C-r>s//<Left>]]
vim.cmd[[nnoremap <unique> <expr> <CR> empty(&buftype) ? ':w<CR>' : '<CR>']]

-- poor man's surround
vim.cmd[[vnoremap _" :call text#surround('"', '"')<CR>]]
vim.cmd[[vnoremap _' :call text#surround("'", "'")<CR>]]
vim.cmd[[vnoremap _( :call text#surround('(', ')')<CR>]]
vim.cmd[[vnoremap _= :call text#surround('`', '`')<CR>]]
vim.cmd[[vnoremap _[ :call text#surround('[', ']')<CR>]]
vim.cmd[[vnoremap _{ :call text#surround('{', '}')<CR>]]

-- gitsigns (<localleader>)
-- See ~/.config/nvim/init.lua

-- nvim_lsp (<localleader>)
-- See ~/.config/nvim/lua/lsp.lua

-- compe (<C->)
vim.cmd[[inoremap <silent><expr> <tab> compe#complete()]]
vim.cmd[[inoremap <silent><expr> <CR>      compe#confirm('<CR>')]]
vim.cmd[[inoremap <silent><expr> <C-q>     compe#close('<C-q>')]]
vim.cmd[[inoremap <silent><expr> <C-f>     compe#scroll({ 'delta': +4 })]]
vim.cmd[[inoremap <silent><expr> <C-d>     compe#scroll({ 'delta': -4 })]]

-- telescope (<leader>)
-- NOTE: keys already mapped: <tab>, $, E, e, ., d, x
utils.map('n', '<leader><leader>', '<cmd>Telescope buffers<CR>')

utils.map('n', '<leader>/', '<cmd>Telescope current_buffer_fuzzy_find<CR>')
utils.map('n', '<leader>?', '<cmd>Telescope live_grep<CR>')
utils.map('n', '<leader>*', '<cmd>Telescope grep_string<CR>')
utils.map('n', '<leader>:', '<cmd>Telescope command_history<CR>')
utils.map('n', '<leader>f', '<cmd>Telescope find_files<CR>')
utils.map('n', '<leader>G', '<cmd>Telescope git_commits<CR>')
utils.map('n', '<leader>g', '<cmd>Telescope git_status<CR>')
utils.map('n', '<leader>h', '<cmd>Telescope help_tags<CR>')
utils.map('n', '<leader>q', '<cmd>Telescope quickfix<CR>')
utils.map('n', '<leader>r', '<cmd>Telescope oldfiles<CR>')
utils.map('n', '<leader>t', '<cmd>Telescope treesitter<CR>')

-- dap (<localleader>)
utils.map('n', '<localleader>db', "<cmd>lua require'dap'.toggle_breakpoint()<CR>")
utils.map('n', '<localleader>dB', "<cmd>lua require'dap'.set_breakpoint(vim.fn.input('Breakpoint condition: '))<CR>")
utils.map('n', '<localleader>dr', "<cmd>lua require'dap'.repl.open()<CR>")
utils.map('n', '<localleader>di', "<cmd>lua require'dap'.step_into()<CR>")
utils.map('n', '<localleader>do', "<cmd>lua require'dap'.step_out()<CR>")
utils.map('n', '<localleader>dn', "<cmd>lua require'dap'.step_over()<CR>")
utils.map('n', '<localleader>dc', "<cmd>lua require'dap'.continue()<CR>")
