-- Sources:
-- * Seth Brown (https://github.com/seth-brown/dotfiles)
-- * Max Horn (https://expectationmax.github.io/2020/NeoVims-Language-Server-Client/)
local utils = { }

local api, cmd, fn, g = vim.api, vim.cmd, vim.fn, vim.g
local scopes = {o = vim.o, b = vim.bo, w = vim.wo}
local fmt = string.format

local lsp = require 'vim.lsp'
local util = require 'vim.lsp.util'
local log = require 'vim.lsp.log'

function utils.map(mode, lhs, rhs, opts)
  local options = {noremap = true}
  if opts then options = vim.tbl_extend('force', options, opts) end
  api.nvim_set_keymap(mode, lhs, rhs, options)
end

function utils.opt(scope, key, value)
  scopes[scope][key] = value
  if scope ~= 'o' then scopes['o'][key] = value end
end

function utils.is_buffer_empty()
    return fn.empty(vim.fn.expand('%:t')) == 1
end

function utils.has_width_gt(cols)
    return fn.winwidth(0) / 2 > cols
end

function utils.tag_lsp(pattern, flags, info)
  -- require and then setlocal tagfunc=v:lua.tagfunc_nvim_lsp
  local result = {}
  local isSearchingFromNormalMode = flags == "c"

  local method
  local params
  if isSearchingFromNormalMode then
    -- Jump to the definition of the symbol under the cursor
    -- when called by CTRL-]
    method = 'textDocument/definition'
    params = util.make_position_params()
  else
    method = 'workspace/symbol'
    params = {}
    if string.find(flags, 'i') then
      params.query = string.sub(pattern, '^\\<', '')
    else
      params.query = pattern
    end
  end
  local client_id_to_results, err = lsp.buf_request_sync(0, method, params, 800)
  if err then
    print('Error when calling tagfunc: ' .. err)
    return result
  end

  for _client_id, results in pairs(client_id_to_results) do
    for i, lsp_result in ipairs(results.result) do
      local name
      local location
      if isSearchingFromNormalMode then
        name = pattern
        location = lsp_result
      else
        name = lsp_result.name
        location = lsp_result.location
      end
      local location_for_tagfunc = {
        name = name,
        filename = vim.uri_to_fname(location.uri),
        cmd = tostring(location.range.start.line + 1)
      }
      table.insert(result, location_for_tagfunc)
    end
  end
  return result
end

return utils
